% Chudik, Pesaran and Yang (2016)
% 
% Data generating process
% 
% Y(i,t) = muY(i) + lambdaY * Y(i,t-1) + [1 - lambdaY] * beta * X(i,t) + U(i,t).
% X(i,t) = [1 - lambdaX] * muX(i) + [1 - lambdaX] * kappaX * Y(i,t-1)
%           + lambdaX * X(i,t-1) + v(i,t)
%
% beta = 0.5, lambdaX = 0.25,
% muX(i) ~ i.i.d. N(1, 1), muY(i) = muX(i) + etaY(i), etaY(i) ~ i.i.d. N(1, 1),
% v(i,t) ~ i.i.d. N(0, sigmav^2(i)), sigmav^2(i) = 0.5 + 0.25 * etav^2(i),
%           etav^2(i) ~ i.i.d. chi^2(2),
% u(i,t) ~ i.i.d. N(0, sigmau^2(i)), sigmau^2(i) = 0.5 + 0.25 * etau^2(i),
%           etau^2(i) ~ i.i.d. chi^2(2),
% lambdaY = 0.4 or 0.8, 
% kappaX = 0, 0.2 or 0.4.

clc
clear
close all

tic

parpool

% Replications
R = 2000;

% Parameters
betaY = 0.5;
lambdaX = 0.25;

lambdaY = 0.4;
kappaX = 0;

% phiY and bY

phiY = lambdaY - 1;
bY = (1 - lambdaY ) * betaY;

% Sample Size
NN = [30; 60; 100; 200; 500; 1000];
TT = [30; 60; 100; 200];

DN = length(NN);
DT = length(TT);

maxN = NN(DN);
maxT = TT(DT);

% Burn-in
B = 100;

% Results

% r: robust, d: Hashem's formula

COEFFEbetaY = zeros(R, DN, DT);
STDFEbetaY = zeros(R, DN, DT);
BIASFEbetaY = zeros(DN, DT);
MSEFEbetaY = zeros(DN, DT);
SIZEFEbetaY = zeros(DN, DT);

COEFFEphiY = zeros(R, DN, DT);
STDFEphiY = zeros(R, DN, DT);
BIASFEphiY = zeros(DN, DT);
MSEFEphiY = zeros(DN, DT);
SIZEFEphiY = zeros(DN, DT);

COEFFEbY = zeros(R, DN, DT);
STDFEbY = zeros(R, DN, DT);
BIASFEbY = zeros(DN, DT);
MSEFEbY = zeros(DN, DT);
SIZEFEbY = zeros(DN, DT);

COEFFEbetaYJ = zeros(R, DN, DT);
STDFEbetaYJ = zeros(R, DN, DT);
BIASFEbetaYJ = zeros(DN, DT);
MSEFEbetaYJ = zeros(DN, DT);
SIZEFEbetaYJ = zeros(DN, DT);

COEFFEphiYJ = zeros(R, DN, DT);
STDFEphiYJ = zeros(R, DN, DT);
BIASFEphiYJ = zeros(DN, DT);
MSEFEphiYJ = zeros(DN, DT);
SIZEFEphiYJ = zeros(DN, DT);

COEFFEbYJ = zeros(R, DN, DT);
STDFEbYJ = zeros(R, DN, DT);
BIASFEbYJ = zeros(DN, DT);
MSEFEbYJ = zeros(DN, DT);
SIZEFEbYJ = zeros(DN, DT);

COEFFETEbetaY = zeros(R, DN, DT);
STDFETEbetaY = zeros(R, DN, DT);
BIASFETEbetaY = zeros(DN, DT);
MSEFETEbetaY = zeros(DN, DT);
SIZEFETEbetaY = zeros(DN, DT);

COEFFETEphiY = zeros(R, DN, DT);
STDFETEphiY = zeros(R, DN, DT);
BIASFETEphiY = zeros(DN, DT);
MSEFETEphiY = zeros(DN, DT);
SIZEFETEphiY = zeros(DN, DT);

COEFFETEbY = zeros(R, DN, DT);
STDFETEbY = zeros(R, DN, DT);
BIASFETEbY = zeros(DN, DT);
MSEFETEbY = zeros(DN, DT);
SIZEFETEbY = zeros(DN, DT);

COEFFETEbetaYJ = zeros(R, DN, DT);
STDFETEbetaYJ = zeros(R, DN, DT);
BIASFETEbetaYJ = zeros(DN, DT);
MSEFETEbetaYJ = zeros(DN, DT);
SIZEFETEbetaYJ = zeros(DN, DT);

COEFFETEphiYJ = zeros(R, DN, DT);
STDFETEphiYJ = zeros(R, DN, DT);
BIASFETEphiYJ = zeros(DN, DT);
MSEFETEphiYJ = zeros(DN, DT);
SIZEFETEphiYJ = zeros(DN, DT);

COEFFETEbYJ = zeros(R, DN, DT);
STDFETEbYJ = zeros(R, DN, DT);
BIASFETEbYJ = zeros(DN, DT);
MSEFETEbYJ = zeros(DN, DT);
SIZEFETEbYJ = zeros(DN, DT);

for r = 1:R
    
    N = maxN;
    T = B + maxT;
    
    % Parameters
    
    LambdaY = ones(N, 1) * lambdaY;
    BetaY = ones(N, 1) * betaY;
    
    SigmaU2 = ones(N, 1) * 0.5 + (randn(N, 1).^2 + randn(N, 1).^2) * 0.25;
    SigmaU = SigmaU2.^0.5;
    
    LambdaX = ones(N, 1) * lambdaX;
    KappaX = ones(N, 1) * kappaX;
    
    CX = ones(N, 1) + randn(N, 1);
    CY = CX + ones(N, 1) + randn(N, 1);
    
    SigmaV2 = ones(N, 1) * 0.5 + (randn(N, 1).^2 + randn(N, 1).^2) * 0.25;
    SigmaV = SigmaV2.^0.5;
    
    % Errors
    
    UY = randn(N, T) .* (SigmaU * ones(1, T));
    VX = randn(N, T) .* (SigmaV * ones(1, T));
    
    % Generating Data
    
    LX = zeros(N, 1);
    LY = zeros(N, 1);
    
    XX = zeros(N, T);
    YY = zeros(N, T);

    for t = 1:T

        X = (ones(N, 1) - LambdaX) .* CX + LambdaX .* LX...
            + (ones(N, 1) - LambdaX) .* KappaX .* LY...
            + VX(:, t);

        Y = CY + LambdaY .* LY + (ones(N, 1) - LambdaY) .* BetaY .* X + UY(:, t);

        XX(:, t) = X;
        YY(:, t) = Y;

        LX = X;
        LY = Y;

    end
    
    LYY = YY(:, B:(T-1));
    YY = YY(:, (B+1):T);
    DYY = YY - LYY;
    XX = XX(:, (B+1):T);
    
    % Estimation
    
    for dn = 1:DN
        
        N = NN(dn);
        
        for dt = 1:DT
            
            T = TT(dt);
            
            Y = YY(1:N, 1:T);
            DY = DYY(1:N, 1:T);
            LY = LYY(1:N, 1:T);
            X = XX(1:N, 1:T);
            
            % FE Estimation
            
% D.Y(i,t) = aY(i) + gammaY * t
%           + phi * Y(i,t-1) + b * X(i,t) + d * D.X(i,t) + E(i,t).
            
            DYm = DY - mean(DY, 2) * ones(1, T);
            LYm = LY - mean(LY, 2) * ones(1, T);
            Xm = X - mean(X, 2) * ones(1, T);
            
            DYmtau = DY - ones(N, 1) * mean(DY, 1) - mean(DY, 2) * ones(1, T) + ones(N, T) * mean(mean(DY));
            LYmtau = LY - ones(N, 1) * mean(LY, 1) - mean(LY, 2) * ones(1, T) + ones(N, T) * mean(mean(LY));
            Xmtau = X - ones(N, 1) * mean(X, 1) - mean(X, 2) * ones(1, T) + ones(N, T) * mean(mean(X));
            
            WMW = zeros(2, 2);
            WMDY = zeros(2, 1);
            
            WMWtau = zeros(2, 2);
            WMDYtau = zeros(2, 1);
            
            for i = 1:N
                
                dYm = DYm(i, :)';
                Wm = [LYm(i, :)', Xm(i, :)'];
                WMW = WMW + Wm' * Wm;
                WMDY = WMDY + Wm' * dYm;
                
                dYmtau = DYmtau(i, :)';
                Wmtau = [LYmtau(i, :)', Xmtau(i, :)'];
                WMWtau = WMWtau + Wmtau' * Wmtau;
                WMDYtau = WMDYtau + Wmtau' * dYmtau;
                
            end
            
            invWMW = WMW^(-1);
            FEest = invWMW * WMDY;
            BetaYFE = -FEest(2) / FEest(1);
            
            COEFFEbetaY(r, dn, dt) = BetaYFE;
            BIASFEbetaY(dn, dt) = BIASFEbetaY(dn, dt) + 100 * (BetaYFE - betaY) / R;
            MSEFEbetaY(dn, dt) = MSEFEbetaY(dn, dt) + 10000 * (BetaYFE - betaY)^2 / R;
            
            COEFFEphiY(r, dn, dt) = FEest(1);
            BIASFEphiY(dn, dt) = BIASFEphiY(dn, dt) + 100 * (FEest(1) - phiY) / R;
            MSEFEphiY(dn, dt) = MSEFEphiY(dn, dt) + 10000 * (FEest(1) - phiY)^2 / R;
            
            COEFFEbY(r, dn, dt) = FEest(2);
            BIASFEbY(dn, dt) = BIASFEbY(dn, dt) + 100 * (FEest(2) - bY) / R;
            MSEFEbY(dn, dt) = MSEFEbY(dn, dt) + 10000 * (FEest(2) - bY)^2 / R;
            
            invWMWtau = WMWtau^(-1);
            FETEest = invWMWtau * WMDYtau;
            BetaYFETE = -FETEest(2) / FETEest(1);
            
            COEFFETEbetaY(r, dn, dt) = BetaYFETE;
            BIASFETEbetaY(dn, dt) = BIASFETEbetaY(dn, dt) + 100 * (BetaYFETE - betaY) / R;
            MSEFETEbetaY(dn, dt) = MSEFETEbetaY(dn, dt) + 10000 * (BetaYFETE - betaY)^2 / R;
            
            COEFFETEphiY(r, dn, dt) = FETEest(1);
            BIASFETEphiY(dn, dt) = BIASFETEphiY(dn, dt) + 100 * (FETEest(1) - phiY) / R;
            MSEFETEphiY(dn, dt) = MSEFETEphiY(dn, dt) + 10000 * (FETEest(1) - phiY)^2 / R;
            
            COEFFETEbY(r, dn, dt) = FETEest(2);
            BIASFETEbY(dn, dt) = BIASFETEbY(dn, dt) + 100 * (FETEest(2) - bY) / R;
            MSEFETEbY(dn, dt) = MSEFETEbY(dn, dt) + 10000 * (FETEest(2) - bY)^2 / R;
            
            % Jackknife
            
            DYma = DY(:, 1:T/2) - mean(DY(:, 1:T/2), 2) * ones(1, T/2);
            DYmb = DY(:, T/2+1:T) - mean(DY(:, T/2+1:T), 2) * ones(1, T/2);
            LYma = LY(:, 1:T/2) - mean(LY(:, 1:T/2), 2) * ones(1, T/2);
            LYmb = LY(:, T/2+1:T) - mean(LY(:, T/2+1:T), 2) * ones(1, T/2);
            Xma = X(:, 1:T/2) - mean(X(:, 1:T/2), 2) * ones(1, T/2);
            Xmb = X(:, T/2+1:T) - mean(X(:, T/2+1:T), 2) * ones(1, T/2);
            
            DYmtaua = DY(:, 1:T/2) - ones(N, 1) * mean(DY(:, 1:T/2), 1) - mean(DY(:, 1:T/2), 2) * ones(1, T/2) + ones(N, T/2) * mean(mean(DY(:, 1:T/2)));
            DYmtaub = DY(:, T/2+1:T) - ones(N, 1) * mean(DY(:, T/2+1:T), 1) - mean(DY(:, T/2+1:T), 2) * ones(1, T/2) + ones(N, T/2) * mean(mean(DY(:, T/2+1:T)));
            LYmtaua = LY(:, 1:T/2) - ones(N, 1) * mean(LY(:, 1:T/2), 1) - mean(LY(:, 1:T/2), 2) * ones(1, T/2) + ones(N, T/2) * mean(mean(LY(:, 1:T/2)));
            LYmtaub = LY(:, T/2+1:T) - ones(N, 1) * mean(LY(:, T/2+1:T), 1) - mean(LY(:, T/2+1:T), 2) * ones(1, T/2) + ones(N, T/2) * mean(mean(LY(:, T/2+1:T)));
            Xmtaua = X(:, 1:T/2) - ones(N, 1) * mean(X(:, 1:T/2), 1) - mean(X(:, 1:T/2), 2) * ones(1, T/2) + ones(N, T/2) * mean(mean(X(:, 1:T/2)));
            Xmtaub = X(:, T/2+1:T) - ones(N, 1) * mean(X(:, T/2+1:T), 1) - mean(X(:, T/2+1:T), 2) * ones(1, T/2) + ones(N, T/2) * mean(mean(X(:, T/2+1:T)));
            
            WMWa = zeros(2, 2);
            WMDYa = zeros(2, 1);
            WMWb = zeros(2, 2);
            WMDYb = zeros(2, 1);
            
            WMWtaua = zeros(2, 2);
            WMDYtaua = zeros(2, 1);
            WMWtaub = zeros(2, 2);
            WMDYtaub = zeros(2, 1);
            
            for i = 1:N
                
                dYma = DYma(i, :)';
                Wma = [LYma(i, :)', Xma(i, :)'];
                WMWa = WMWa + Wma' * Wma;
                WMDYa = WMDYa + Wma' * dYma;
                
                dYmb = DYmb(i, :)';
                Wmb = [LYmb(i, :)', Xmb(i, :)'];
                WMWb = WMWb + Wmb' * Wmb;
                WMDYb = WMDYb + Wmb' * dYmb;
                
                dYmtaua = DYmtaua(i, :)';
                Wmtaua = [LYmtaua(i, :)', Xmtaua(i, :)'];
                WMWtaua = WMWtaua + Wmtaua' * Wmtaua;
                WMDYtaua = WMDYtaua + Wmtaua' * dYmtaua;
                
                dYmtaub = DYmtaub(i, :)';
                Wmtaub = [LYmtaub(i, :)', Xmtaub(i, :)'];
                WMWtaub = WMWtaub + Wmtaub' * Wmtaub;
                WMDYtaub = WMDYtaub + Wmtaub' * dYmtaub;
                
            end
            
            invWMWa = WMWa^(-1);
            FEesta = invWMWa * WMDYa;
            invWMWb = WMWb^(-1);
            FEestb = invWMWb * WMDYb;
            FEestJ = 2 * FEest - 0.5 * FEesta - 0.5 * FEestb;
            BetaYFEJ = -FEestJ(2) / FEestJ(1);
            
            COEFFEbetaYJ(r, dn, dt) = BetaYFEJ;
            BIASFEbetaYJ(dn, dt) = BIASFEbetaYJ(dn, dt) + 100 * (BetaYFEJ - betaY) / R;
            MSEFEbetaYJ(dn, dt) = MSEFEbetaYJ(dn, dt) + 10000 * (BetaYFEJ - betaY)^2 / R;
            
            COEFFEphiYJ(r, dn, dt) = FEestJ(1);
            BIASFEphiYJ(dn, dt) = BIASFEphiYJ(dn, dt) + 100 * (FEestJ(1) - phiY) / R;
            MSEFEphiYJ(dn, dt) = MSEFEphiYJ(dn, dt) + 10000 * (FEestJ(1) - phiY)^2 / R;
            
            COEFFEbYJ(r, dn, dt) = FEestJ(2);
            BIASFEbYJ(dn, dt) = BIASFEbYJ(dn, dt) + 100 * (FEestJ(2) - bY) / R;
            MSEFEbYJ(dn, dt) = MSEFEbYJ(dn, dt) + 10000 * (FEestJ(2) - bY)^2 / R;
            
            invWMWtaua = WMWtaua^(-1);
            FETEesta = invWMWtaua * WMDYtaua;
            invWMWtaub = WMWtaub^(-1);
            FETEestb = invWMWtaub * WMDYtaub;
            FETEestJ = 2 * FETEest - 0.5 * FETEesta - 0.5 * FETEestb;
            BetaYFETEJ = -FETEestJ(2) / FETEestJ(1);
            
            COEFFETEbetaYJ(r, dn, dt) = BetaYFETEJ;
            BIASFETEbetaYJ(dn, dt) = BIASFETEbetaYJ(dn, dt) + 100 * (BetaYFETEJ - betaY) / R;
            MSEFETEbetaYJ(dn, dt) = MSEFETEbetaYJ(dn, dt) + 10000 * (BetaYFETEJ - betaY)^2 / R;
            
            COEFFETEphiYJ(r, dn, dt) = FETEestJ(1);
            BIASFETEphiYJ(dn, dt) = BIASFETEphiYJ(dn, dt) + 100 * (FETEestJ(1) - phiY) / R;
            MSEFETEphiYJ(dn, dt) = MSEFETEphiYJ(dn, dt) + 10000 * (FETEestJ(1) - phiY)^2 / R;
            
            COEFFETEbYJ(r, dn, dt) = FETEestJ(2);
            BIASFETEbYJ(dn, dt) = BIASFETEbYJ(dn, dt) + 100 * (FETEestJ(2) - bY) / R;
            MSEFETEbYJ(dn, dt) = MSEFETEbYJ(dn, dt) + 10000 * (FETEestJ(2) - bY)^2 / R;
            
            % Variance
            
            WuuW = zeros(2, 2);
            WuuWJ = zeros(2, 2);
            
            WuuWtau = zeros(2, 2);
            WuuWtauJ = zeros(2, 2);
            
            DYmma = DY(:, 1:T/2) - 2 * mean(DY, 2) * ones(1, T/2) + mean(DY(:, 1:T/2), 2) * ones(1, T/2);
            DYmmb = DY(:, T/2+1:T) - 2 * mean(DY, 2) * ones(1, T/2) + mean(DY(:, T/2+1:T), 2) * ones(1, T/2);
            DYmm = [DYmma, DYmmb];
            LYmma = LY(:, 1:T/2) - 2 * mean(LY, 2) * ones(1, T/2) + mean(LY(:, 1:T/2), 2) * ones(1, T/2);
            LYmmb = LY(:, T/2+1:T) - 2 * mean(LY, 2) * ones(1, T/2) + mean(LY(:, T/2+1:T), 2) * ones(1, T/2);
            LYmm = [LYmma, LYmmb];
            Xmma = X(:, 1:T/2) - 2 * mean(X, 2) * ones(1, T/2) + mean(X(:, 1:T/2), 2) * ones(1, T/2);
            Xmmb = X(:, T/2+1:T) - 2 * mean(X, 2) * ones(1, T/2) + mean(X(:, T/2+1:T), 2) * ones(1, T/2);
            Xmm = [Xmma, Xmmb];
            
            DYmmtaua = DY(:, 1:T/2) - ones(N, 1) * mean(DY(:, 1:T/2), 1) - 2 * mean(DY, 2) * ones(1, T/2) + mean(DY(:, 1:T/2), 2) * ones(1, T/2)...
                + 2 * ones(N, T/2) * mean(mean(DY)) - ones(N, T/2) * mean(mean(DY(:, 1:T/2)));
            DYmmtaub = DY(:, T/2+1:T) - ones(N, 1) * mean(DY(:, T/2+1:T), 1) - 2 * mean(DY, 2) * ones(1, T/2) + mean(DY(:, T/2+1:T), 2) * ones(1, T/2)...
                + 2 * ones(N, T/2) * mean(mean(DY)) - ones(N, T/2) * mean(mean(DY(:, T/2+1:T)));
            DYmmtau = [DYmmtaua, DYmmtaub];
            LYmmtaua = LY(:, 1:T/2) - ones(N, 1) * mean(LY(:, 1:T/2), 1) - 2 * mean(LY, 2) * ones(1, T/2) + mean(LY(:, 1:T/2), 2) * ones(1, T/2)...
                + 2 * ones(N, T/2) * mean(mean(LY)) - ones(N, T/2) * mean(mean(LY(:, 1:T/2)));
            LYmmtaub = LY(:, T/2+1:T) - ones(N, 1) * mean(LY(:, T/2+1:T), 1) - 2 * mean(LY, 2) * ones(1, T/2) + mean(LY(:, T/2+1:T), 2) * ones(1, T/2)...
                + 2 * ones(N, T/2) * mean(mean(LY)) - ones(N, T/2) * mean(mean(LY(:, T/2+1:T)));
            LYmmtau = [LYmmtaua, LYmmtaub];
            Xmmtaua = X(:, 1:T/2) - ones(N, 1) * mean(X(:, 1:T/2), 1) - 2 * mean(X, 2) * ones(1, T/2) + mean(X(:, 1:T/2), 2) * ones(1, T/2)...
                + 2 * ones(N, T/2) * mean(mean(X)) - ones(N, T/2) * mean(mean(X(:, 1:T/2)));
            Xmmtaub = X(:, T/2+1:T) - ones(N, 1) * mean(X(:, T/2+1:T), 1) - 2 * mean(X, 2) * ones(1, T/2) + mean(X(:, T/2+1:T), 2) * ones(1, T/2)...
                + 2 * ones(N, T/2) * mean(mean(X)) - ones(N, T/2) * mean(mean(X(:, T/2+1:T)));
            Xmmtau = [Xmmtaua, Xmmtaub];
            
            for i = 1:N
                
                dYm = DYm(i, :)';
                Wm = [LYm(i, :)', Xm(i, :)'];
                Wmm = [LYmm(i, :)', Xmm(i, :)'];
                uFE = dYm - Wm * FEest;
                uFEJ = dYm - Wm * FEestJ;
                
                for t = 1:T
                    
                    WuuW = WuuW + uFE(t)^2 * Wm(t, :)'* Wm(t, :);
                    WuuWJ = WuuWJ + uFEJ(t)^2 * Wmm(t, :)'* Wmm(t, :);
                    
                end
                
                dYmtau = DYmtau(i, :)';
                Wmtau = [LYmtau(i, :)', Xmtau(i, :)'];
                Wmmtau = [LYmmtau(i, :)', Xmmtau(i, :)'];
                uFETE = dYmtau - Wmtau * FETEest;
                uFETEJ = dYmtau - Wmtau * FETEestJ;
                
                for t = 1:T
                    
                    WuuWtau = WuuWtau + uFETE(t)^2 * Wmtau(t, :)' * Wmtau(t, :);
                    WuuWtauJ = WuuWtauJ + uFETEJ(t)^2 * Wmmtau(t, :)' * Wmmtau(t, :);
                    
                end
                
            end
            
            AvarFE = invWMW * WuuW * invWMW;
            BetaYFEstd = sqrt([FEest(2) / (FEest(1))^2, -1 / FEest(1)] * AvarFE *...
                [FEest(2) / (FEest(1))^2; -1 / FEest(1)]);
                        
            AvarFEJ = invWMW * WuuWJ * invWMW;
            BetaYFEstdJ = sqrt([FEestJ(2) / (FEestJ(1))^2, -1 / FEestJ(1)] * AvarFEJ *...
                [FEestJ(2) / (FEestJ(1))^2; -1 / FEestJ(1)]);
            
            AvarFETE = invWMWtau * WuuWtau * invWMWtau;
            BetaYFETEstd = sqrt([FETEest(2) / (FETEest(1))^2, -1 / FETEest(1)] * AvarFETE *...
                [FETEest(2) / (FETEest(1))^2; -1 / FETEest(1)]);
                        
            AvarFETEJ = invWMWtau * WuuWtauJ * invWMWtau;
            BetaYFETEstdJ = sqrt([FETEestJ(2) / (FETEestJ(1))^2, -1 / FETEestJ(1)] * AvarFETEJ *...
                [FETEestJ(2) / (FETEestJ(1))^2; -1 / FETEestJ(1)]);
            
            STDFEbetaY(r, dn, dt) = BetaYFEstd;
            STDFEphiY(r, dn, dt) = sqrt(AvarFE(1, 1));
            STDFEbY(r, dn, dt) = sqrt(AvarFE(2, 2));
            
            SIZEFEbetaY(dn, dt) = SIZEFEbetaY(dn, dt) + 100 * (abs((BetaYFE - betaY) / BetaYFEstd) > 1.96) / R;
            SIZEFEphiY(dn, dt) = SIZEFEphiY(dn, dt) + 100 * (abs((FEest(1) - phiY) / sqrt(AvarFE(1, 1))) > 1.96) / R;
            SIZEFEbY(dn, dt) = SIZEFEbY(dn, dt) + 100 * (abs((FEest(2) - bY) / sqrt(AvarFE(2, 2))) > 1.96) / R;
            
            STDFEbetaYJ(r, dn, dt) = BetaYFEstdJ;
            STDFEphiYJ(r, dn, dt) = sqrt(AvarFEJ(1, 1));
            STDFEbYJ(r, dn, dt) = sqrt(AvarFEJ(2, 2));
            
            SIZEFEbetaYJ(dn, dt) = SIZEFEbetaYJ(dn, dt) + 100 * (abs((BetaYFEJ - betaY) / BetaYFEstdJ) > 1.96) / R;
            SIZEFEphiYJ(dn, dt) = SIZEFEphiYJ(dn, dt) + 100 * (abs((FEestJ(1) - phiY) / sqrt(AvarFEJ(1, 1))) > 1.96) / R;
            SIZEFEbYJ(dn, dt) = SIZEFEbYJ(dn, dt) + 100 * (abs((FEestJ(2) - bY) / sqrt(AvarFEJ(2, 2))) > 1.96) / R;
            
            STDFETEbetaY(r, dn, dt) = BetaYFETEstd;
            STDFETEphiY(r, dn, dt) = sqrt(AvarFETE(1, 1));
            STDFETEbY(r, dn, dt) = sqrt(AvarFETE(2, 2));
            
            SIZEFETEbetaY(dn, dt) = SIZEFETEbetaY(dn, dt) + 100 * (abs((BetaYFETE - betaY) / BetaYFETEstd) > 1.96) / R;
            SIZEFETEphiY(dn, dt) = SIZEFETEphiY(dn, dt) + 100 * (abs((FETEest(1) - phiY) / sqrt(AvarFETE(1, 1))) > 1.96) / R;
            SIZEFETEbY(dn, dt) = SIZEFETEbY(dn, dt) + 100 * (abs((FETEest(2) - bY) / sqrt(AvarFETE(2, 2))) > 1.96) / R;
            
            STDFETEbetaYJ(r, dn, dt) = BetaYFETEstdJ;
            STDFETEphiYJ(r, dn, dt) = sqrt(AvarFETEJ(1, 1));
            STDFETEbYJ(r, dn, dt) = sqrt(AvarFETEJ(2, 2));
            
            SIZEFETEbetaYJ(dn, dt) = SIZEFETEbetaYJ(dn, dt) + 100 * (abs((BetaYFETEJ - betaY) / BetaYFETEstdJ) > 1.96) / R;
            SIZEFETEphiYJ(dn, dt) = SIZEFETEphiYJ(dn, dt) + 100 * (abs((FETEestJ(1) - phiY) / sqrt(AvarFETEJ(1, 1))) > 1.96) / R;
            SIZEFETEbYJ(dn, dt) = SIZEFETEbYJ(dn, dt) + 100 * (abs((FETEestJ(2) - bY) / sqrt(AvarFETEJ(2, 2))) > 1.96) / R;
            
        end
        
    end
    
    r
    toc
    
end

clearvars -except savefile g1Y g2Y lambdaY kappaX...
    COEFFEbetaY STDFEbetaY BIASFEbetaY MSEFEbetaY SIZEFEbetaY...
    COEFFEphiY STDFEphiY BIASFEphiY MSEFEphiY SIZEFEphiY...
    COEFFEbY STDFEbY BIASFEbY MSEFEbY SIZEFEbY...
    COEFFEbetaYJ STDFEbetaYJ BIASFEbetaYJ MSEFEbetaYJ SIZEFEbetaYJ...
    COEFFEphiYJ STDFEphiYJ BIASFEphiYJ MSEFEphiYJ SIZEFEphiYJ...
    COEFFEbYJ STDFEbYJ BIASFEbYJ MSEFEbYJ SIZEFEbYJ...
    COEFFETEbetaY STDFETEbetaY BIASFETEbetaY MSEFETEbetaY SIZEFETEbetaY...
    COEFFETEphiY STDFETEphiY BIASFETEphiY MSEFETEphiY SIZEFETEphiY...
    COEFFETEbY STDFETEbY BIASFETEbY MSEFETEbY SIZEFETEbY...
    COEFFETEbetaYJ STDFETEbetaYJ BIASFETEbetaYJ MSEFETEbetaYJ SIZEFETEbetaYJ...
    COEFFETEphiYJ STDFETEphiYJ BIASFETEphiYJ MSEFETEphiYJ SIZEFETEphiYJ...
    COEFFETEbYJ STDFETEbYJ BIASFETEbYJ MSEFETEbYJ SIZEFETEbYJ

RMSEFEbetaY = MSEFEbetaY.^0.5;
RMSEFEphiY = MSEFEphiY.^0.5;
RMSEFEbY = MSEFEbY.^0.5;
RMSEFEbetaYJ = MSEFEbetaYJ.^0.5;
RMSEFEphiYJ = MSEFEphiYJ.^0.5;
RMSEFEbYJ = MSEFEbYJ.^0.5;
RMSEFETEbetaY = MSEFETEbetaY.^0.5;
RMSEFETEphiY = MSEFETEphiY.^0.5;
RMSEFETEbY = MSEFETEbY.^0.5;
RMSEFETEbetaYJ = MSEFETEbetaYJ.^0.5;
RMSEFETEphiYJ = MSEFETEphiYJ.^0.5;
RMSEFETEbYJ = MSEFETEbYJ.^0.5;

toc

lambdaY
kappaX

BIASFEbetaY
BIASFEphiY
BIASFEbY

BIASFEbetaYJ
BIASFEphiYJ
BIASFEbYJ

BIASFETEbetaY
BIASFETEphiY
BIASFETEbY

BIASFETEbetaYJ
BIASFETEphiYJ
BIASFETEbYJ

RMSEFEbetaY
RMSEFEphiY
RMSEFEbY

RMSEFEbetaYJ
RMSEFEphiYJ
RMSEFEbYJ

RMSEFETEbetaY
RMSEFETEphiY
RMSEFETEbY

RMSEFETEbetaYJ
RMSEFETEphiYJ
RMSEFETEbYJ

SIZEFEbetaY
SIZEFEphiY
SIZEFEbY

SIZEFEbetaYJ
SIZEFEphiYJ
SIZEFEbYJ

SIZEFETEbetaY
SIZEFETEphiY
SIZEFETEbY

SIZEFETEbetaYJ
SIZEFETEphiYJ
SIZEFETEbYJ

delete(gcp)

toc